/******************************************************************************
  MMC_Driver */
#include "TFR_objects.h"
#include "TFR_resources.h"
#include "__Time.h"
#include "USB_Driver.h"
#include "MMC_Driver.h"
//#include "printf.h"

// MMC Chip Select connection
sbit Mmc_Chip_Select_Direction at GPIO_PORTD_DIR.B1;
sbit Mmc_Chip_Select           at GPIO_PORTD_DATA.B1;

//Variables
char filename[14] = "TFxxxxxx.TXT";  // File name

char err_txt[20] = "FAT16 not found";
char Buffer[512];
unsigned long  size;

char OpenDate[16];
char OpenTime[16];

extern TimeStruct ts1;
extern void UpdateStatus(char * msg);
extern int Interval;
extern int IntervalTable[];


// This function opens a file and writes the header information
void StartRecording() {
     char Yr[6];
     char Mo[4];
     char Da[4];
     char Hr[4];
     char Mi[4];
     char Se[4];
     
     unsigned int Stat;
     int IntervalTime;

     sprintf(Yr,"%04d",ts1.yy);
     sprintf(Mo,"%02d",ts1.mo);
     sprintf(Da,"%02d",ts1.md);
     sprintf(Hr,"%02d",ts1.hh);
     sprintf(Mi,"%02d",ts1.mn);
     sprintf(Se,"%02d",ts1.ss);

     filename[0] = Yr[0];
     filename[1] = Yr[1];
     filename[2] = Yr[2];
     filename[3] = Yr[3];
     filename[4] = Mo[0];
     filename[5] = Mo[1];
     filename[6] = Da[0];
     filename[7] = Da[1];

     if (Mmc_Fat_Exists(&filename)) {
         Stat = Mmc_Fat_Assign(&filename, 0);
         Mmc_Fat_Append();
     } else {
         Stat = Mmc_Fat_Assign(&filename, 0xA0);  // Find existing file or create a new one
         Mmc_Fat_Set_File_Date(ts1.yy, ts1.mo, ts1.md, ts1.hh, ts1.mn, ts1.ss); // Set file date & time info
         Mmc_Fat_Rewrite();
     }

     // This convolution is necessary because sprintf doesn't work properly
     strcpy(Buffer,"[Header]\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Create and write the date of the file
     strcpy(OpenDate,Yr);
     strcat(OpenDate,"/");
     strcat(OpenDate,Mo);
     strcat(OpenDate,"/");
     strcat(OpenDate,Da);
     
     strcpy(Buffer,"OpenDate=");
     strcat(Buffer,OpenDate);
     strcat(Buffer,"\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Create and write the exact time the file was opened
     strcpy(OpenTime,Hr);
     strcat(OpenTime,":");
     strcat(OpenTime,Mi);
     strcat(OpenTime,":");
     strcat(OpenTime,Se);

     strcpy(Buffer,"OpenTime=");
     strcat(Buffer,OpenTime);
     strcat(Buffer,"\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Record the site number
     strcpy(Buffer,"Site=");
     strcat(Buffer,Start_Label3.Caption);
     strcat(Buffer,"\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Record the interval for the file records
     IntervalTime = IntervalTable[Interval];
     sprintf(Buffer,"Interval=%d\n",IntervalTime);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Header for the records
     strcpy(Buffer,"[Items]\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
}

// This function writes the counters to the file
void RecordOnInterval() {
     char Hr[4];
     char Mi[4];
     char Se[4];
     
     sprintf(Hr,"%02d",ts1.hh);
     sprintf(Mi,"%02d",ts1.mn);
     sprintf(Se,"%02d",ts1.ss);

     // This convolution is necessary because sprintf doesn't work properly
     // First write the time this interval ended
     strcpy(Buffer,"EndingAt=");
     strcat(Buffer,Hr);
     strcat(Buffer,":");
     strcat(Buffer,Mi);
     strcat(Buffer,":");
     strcat(Buffer,Se);
     strcat(Buffer,"\n");
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     
     // Now write all the counters
     sprintf(Buffer,"Spare1=%d\n",Spare1);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NPC=%d\n",NPC);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare2=%d\n",Spare2);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NHR=%d\n",NHR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NHT=%d\n",NHT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NHL=%d\n",NHL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NTR=%d\n",NTR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NTT=%d\n",NTT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NTL=%d\n",NTL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NCR=%d\n",NCR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NCT=%d\n",NCT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"NCL=%d\n",NCL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
//--------------------------------------------//
     sprintf(Buffer,"SCL=%d\n",SCL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SCT=%d\n",SCT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SCR=%d\n",SCR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"STL=%d\n",STL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"STT=%d\n",STT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"STR=%d\n",STR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SHL=%d\n",SHL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SHT=%d\n",SHT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SHR=%d\n",SHR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare6=%d\n",Spare6);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"SPC=%d\n",SPC);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare5=%d\n",Spare5);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
//--------------------------------------------//
     sprintf(Buffer,"Spare8=%d\n",Spare8);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WHL=%d\n",WHL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WTL=%d\n",WTL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WCL=%d\n",WCL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WPC=%d\n",WPC);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WHT=%d\n",WHT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WTT=%d\n",WTT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WCT=%d\n",WCT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare7=%d\n",Spare7);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WHR=%d\n",WHR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WTR=%d\n",WTR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"WCR=%d\n",WCR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
//--------------------------------------------//
     sprintf(Buffer,"ECR=%d\n",ECR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"ETR=%d\n",ETR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"EHR=%d\n",EHR);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare3=%d\n",Spare3);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"ECT=%d\n",ECT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"ETT=%d\n",ETT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"EHT=%d\n",EHT);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"EPC=%d\n",EPC);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"ECL=%d\n",ECL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"ETL=%d\n",ETL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"EHL=%d\n",EHL);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));
     sprintf(Buffer,"Spare4=%d\n",Spare4);
     Mmc_Fat_Write(&Buffer,strlen(Buffer));

     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Spare1,NPC,Spare2,NHR,NHT,NHL,NTR,NTT,NTL,NCR,NCT,NCL);  // Doesn't work
     //strcpy(Buffer,"How now brown cow?\n");  // Works
     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Spare1,NPC,Spare2,NHR,NHT,NHL,NTR,NTT,NTL,NCR);  // 10 values works
     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Spare1,NPC,Spare2,NHR,NHT,NHL,NTR,NTT,NTL,NCR,NCT);  // 11 values or more doesn't work

     //Mmc_Fat_Write(&Buffer,strlen(Buffer));
     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",SCL,SCT,SCR,STL,STT,STR,SHL,SHT,SHR,Spare6,SPC,Spare5);  // Doesn't work
     //strcpy(Buffer,"Same way black bull.\n");  // Works
     //Mmc_Fat_Write(&Buffer,strlen(Buffer));
     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Spare8,WHL,WTL,WCL,WPC,WHT,WTT,WCT,Spare7,WHR,WTR,WCR);  // Doesn't work
     //Mmc_Fat_Write(&Buffer,strlen(Buffer));
     //sprintf(Buffer,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",ECR,ETR,EHR,Spare3,ECT,ETT,EHT,EPC,ECL,ETL,EHL,Spare4);  // Doesn't work
     //Mmc_Fat_Write(&Buffer,strlen(Buffer));

     ClearCounters();
}

// This function writes any remaining couter values and closes the file
void StopRecording() {
     RecordOnInterval();  // Record any remaining counts
     Mmc_Fat_Close();  // Close the file
}

void ClearCounters() {
     Spare1 = 0;
     NPC = 0;
     Spare2 = 0;
     NHR = 0;
     NHT = 0;
     NHL = 0;
     NTR = 0;
     NTT = 0;
     NTL = 0;
     NCR = 0;
     NCT = 0;
     NCL = 0;
     
     SCL = 0;
     SCT = 0;
     SCR = 0;
     STL = 0;
     STT = 0;
     STR = 0;
     SHL = 0;
     SHT = 0;
     SHR = 0;
     Spare6 = 0;
     SPC = 0;
     Spare5 = 0;
     
     Spare8 = 0;
     WHL = 0;
     WTL = 0;
     WCL = 0;
     WPC = 0;
     WHT = 0;
     WTT = 0;
     WCT = 0;
     Spare7 = 0;
     WHR = 0;
     WTR = 0;
     WCR = 0;
     
     ECR = 0;
     ETR = 0;
     EHR = 0;
     Spare3 = 0;
     ECT = 0;
     ETT = 0;
     EHT = 0;
     EPC = 0;
     ECL = 0;
     ETL = 0;
     EHL = 0;
     Spare4 = 0;
}
