/**************************************************************************************************
  Timer */
#include "TFR_objects.h"
#include "__Time.h"
#include "MMC_Driver.h"


// Variables
int TimerFlag = 0;  // Set by interrupt
TimeStruct ts1;
long Epoch;
int IntervalTable[] = {0,1,5,10,15,30,60};

extern int DisplayFlag;    // Display data and time if on
extern int Interval;       // Tracks interval button set
extern void UpdateDate();  // Date display
extern void UpdateTime();  // Time display
extern void UpdateSite();
extern char filename[];    // Defined in MMC_Driver.c


// Timer Interrupt
void Timer0A_interrupt() iv IVT_INT_TIMER0A {
     TIMER_ICR_TATOCINT_bit = 1;         // Clear time-out timer A interrupt
     Epoch += 1;                         // Increment Date/Time by 1 second
     TimerFlag = 1;                      // Set timer flag
}

// Initialization
void TimerInit() {

     SYSCTL_RCGC1_TIMER0_bit = 1;        // Enable clock gating for timer module 0

     TIMER_CTL_TAEN_bit = 0;             // Disable timer
     TIMER0_CFG = 0;                     // Set 32-bit timer configuration
     TIMER0_TAMR |= 2;                   // Set periodic mode
     TIMER0_TAILR = Get_Fosc_kHz()*1000; // Set interval load
     NVIC_IntEnable(IVT_INT_TIMER0A);    // Enable timer interrupt
     TIMER_IMR_TATOIM_bit = 1;           // Enable time-out timer A interrupt
     TIMER_CTL_TAEN_bit = 1;             // Enable timer A

     ts1.ss = 0;  // Cold Start values
     ts1.mn = 0;
     ts1.hh = 0;
     ts1.md = 28;
     ts1.wd = 3;
     ts1.mo = 6;
     ts1.yy = 2012;
  
     Epoch = Time_dateToEpoch(&ts1);
}

// Timer processing is done if timer flag is set
void TimerCheck() {
     if (TimerFlag == 1) {
         TimerFlag = 0;  // Reset timer flag
         Time_epochToDate(Epoch, &ts1);
         
         // Update Date and Time only if Start Screen is showing
         if (DisplayFlag == 1) {
             UpdateDate();
             UpdateTime();
         }
     
         // If on Interval boundary and Start Button is pushed
         if (Interval > 0) {
             if (((ts1.mn % IntervalTable[Interval])==0) && (ts1.ss==0) && (Start_Button11.Gradient==0)) {
                 RecordOnInterval();  // Record counters to flash card
             }
         }
     }
}