/******************************************************************************
  USB_Driver */
#include "TFR_objects.h"
#include "USB_Driver.h"
#include "MMC_Driver.h"

// Variables
char readbuff[64];
char writebuff[64];

tx_q0_struct txqueue0;        // Transmit buffer FIFO and input/output pointers



// Initialization
void USB_Init() {
     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
}

/******************************************************************************
  This routine initializes the transmit buffer FIFO in and out pointers */
void USB_TxQueueInit(void) {
     txqueue0.in = 0;
     txqueue0.out = 0;
}

/******************************************************************************
  This routine makes an entry into the transmit FIFO and updates the in pointer
  to the next available location. */
void USB_PutQueue(char *msg) {
     txqueue0.q[txqueue0.in++] = msg;
     if (txqueue0.in > (txqsize0-1))
         txqueue0.in = 0;
}

/******************************************************************************
  This routine checks the transmit buffer FIFO and, if it is not empty, sends
  the buffer to USB. If the buffer was successfully sent, the out pointer is
  updated to the next available location. */

// Call this function in main loop
void USB_CheckOut(void) {
     if (txqueue0.in != txqueue0.out) {        // Look for a pointer in the FIFO
          if (HID_Write(txqueue0.q[txqueue0.out],64) != 0) {
              txqueue0.out++;
              if (txqueue0.out > (txqsize0-1))        // If past the end of array
                  txqueue0.out = 0;        // set pointer to start of array
          }
     }
}

/******************************************************************************
  This routine checks USB for a receive message then decodes the first
  character for a command. */

// Call this function in main loop
void USB_CheckIn(void) {
     if (HID_Read() != 0) {
          switch (readbuff[0]) {
              case 'a':
                  // Command execution code
                  USB_PutQueue("Command a received");
                  break;
              case 'b':
                  // Command execution code
                  USB_PutQueue("Command b received");

                  //USB_PutQueue("ACK");
                  break;
              // Etc.
              
              default:
                  USB_PutQueue("Unknown command");
                  break;
          }
     }
}